/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlayout.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qstringlist.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <qlabel.h>

#include "kbibtexeditlistbox.h"

KBibTeXEditListBox::KBibTeXEditListBox( const QString& textLabel, QWidget *parent, const char *name ) : QWidget( parent, name )
{
    QGridLayout * layout = new QGridLayout( this, 7, 2, 0, 4 );
    layout->setRowStretch( layout->numRows() - 1, 1 );

    label = new QLabel( textLabel, this, "qeditlistbox_label" );
    layout->addMultiCellWidget( label, 0, 0, 0, 1 );
    lineEditInput = new QLineEdit( this, "qeditlistbox_lineeditinput" );
    layout->addWidget( lineEditInput, 1, 0 );
    label->setBuddy( lineEditInput );

    pushButtonAdd = new QPushButton( this, "qeditlistbox_pushbuttonadd" );
    layout->addWidget( pushButtonAdd, 1, 1 );
    pushButtonAdd->setEnabled( false );

    listBox = new QListBox( this, "qeditlistbox_listbox" );
    layout->addMultiCellWidget( listBox, 2, 6, 0, 0 );

    pushButtonModify = new QPushButton( this, "qeditlistbox_pushbuttonmodify" );
    layout->addWidget( pushButtonModify, 2, 1 );
    pushButtonDelete = new QPushButton( this, "qeditlistbox_pushbuttondelete" );
    layout->addWidget( pushButtonDelete, 3, 1 );
    pushButtonUp = new QPushButton( this, "qeditlistbox_pushbuttonnewup" );
    layout->addWidget( pushButtonUp, 4, 1 );
    pushButtonDown = new QPushButton( this, "qeditlistbox_pushbuttonnewdown" );
    layout->addWidget( pushButtonDown, 5, 1 );

    setFocusProxy( lineEditInput );

    updateUI();

    connect( lineEditInput, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotLineEditTextChanged( ) ) );
    connect( pushButtonAdd, SIGNAL( clicked() ), this, SLOT( slotPushButtonAddClicked() ) );
    connect( pushButtonModify, SIGNAL( clicked() ), this, SLOT( slotPushButtonModifyClicked() ) );
    connect( pushButtonDelete, SIGNAL( clicked() ), this, SLOT( slotPushButtonDeleteClicked() ) );
    connect( pushButtonUp, SIGNAL( clicked() ), this, SLOT( slotPushButtonUpClicked() ) );
    connect( pushButtonDown, SIGNAL( clicked() ), this, SLOT( slotPushButtonDownClicked() ) );
    connect( listBox, SIGNAL( highlighted( int ) ), this, SLOT( slotListBoxHighlighted( ) ) );
}

QStringList KBibTeXEditListBox::lines() const
{
    QListBoxItem * item = listBox->firstItem();
    QStringList items;
    while ( item )
    {
        items << item->text();
        item = item->next();
    }

    return items;
}

void KBibTeXEditListBox::setLines( const QStringList &lines )
{
    listBox->clear();
    listBox->insertStringList( lines );
}

void KBibTeXEditListBox::setButtonText( buttons button, const QString& text )
{
    switch ( button )
    {
    case Add: pushButtonAdd->setText( text ); break;
    case Modify: pushButtonModify->setText( text ); break;
    case Delete: pushButtonDelete->setText( text ); break;
    case Up: pushButtonUp->setText( text ); break;
    case Down: pushButtonDown->setText( text ); break;
    }
}

void KBibTeXEditListBox::slotLineEditTextChanged( )
{
    updateUI();
}

void KBibTeXEditListBox::slotPushButtonAddClicked()
{
    int index = listBox->currentItem();
    listBox->insertItem( lineEditInput->text() );
    listBox->setSelected( index, FALSE );
    listBox->setCurrentItem( -1 );
    lineEditInput->setText( "" );
}

void KBibTeXEditListBox::slotPushButtonModifyClicked()
{
    QString text = lineEditInput->text();
    int index = listBox->currentItem();
    listBox->removeItem( index );
    listBox->insertItem( text, index );
    listBox->setCurrentItem( index );
}

void KBibTeXEditListBox::slotPushButtonDeleteClicked()
{
    int index = listBox->currentItem();
    listBox->removeItem( index );
    //listBox->setCurrentItem( ( index - 1 ) < 0 ? 0 : index - 1 );
    //listBox->setSelected( index - 1, TRUE );
}

void KBibTeXEditListBox::slotPushButtonUpClicked()
{
    QString text = listBox->currentText();
    int index = listBox->currentItem();
    listBox->removeItem( index );
    listBox->insertItem( text, index - 1 );
    listBox->setCurrentItem( index - 1 );
}

void KBibTeXEditListBox::slotPushButtonDownClicked()
{
    QString text = listBox->currentText();
    int index = listBox->currentItem();
    listBox->removeItem( index );
    listBox->insertItem( text, index + 1 );
    listBox->setCurrentItem( index + 1 );
}

void KBibTeXEditListBox::slotListBoxHighlighted( )
{
    if ( listBox->isSelected( listBox->currentItem() ) )
        lineEditInput->setText( listBox->currentText() );
    else
        lineEditInput->setText( "" );
    updateUI();
}

void KBibTeXEditListBox::updateUI()
{
    int index = listBox->currentItem();
    bool curIsSel = listBox->isSelected( listBox->currentItem() );

    pushButtonAdd->setEnabled( !lineEditInput->text().isEmpty() );
    pushButtonModify->setEnabled( !lineEditInput->text().isEmpty() && curIsSel );
    pushButtonDelete->setEnabled( curIsSel && index > -1 );
    pushButtonUp->setEnabled( curIsSel && index > 0 );
    pushButtonDown->setEnabled( curIsSel && index > -1 && index < ( int ) ( listBox->count() - 1 ) );
}

#include "kbibtexeditlistbox.moc"
