// $Id$

/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qlabel.h>
#include <qtabwidget.h>
#include <qspinbox.h>
#include <qregexp.h>
#include <qcheckbox.h>
#include <qstringlist.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qsplitter.h>
#include <qfont.h>
#include <qobjectlist.h>
#include <qtextedit.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <kcombobox.h>
#include <keditlistbox.h>
#include <kpushbutton.h>
#include <klistview.h>
#include <klineedit.h>
#include <ktextedit.h>
#include <kmessagebox.h>
#include <kiconloader.h>

#include <settings.h>
#include <bibtexfile.h>
#include <bibtexentry.h>
#include <kbibtexfieldlistviewitem.h>
#include <kbibtexselectstring.h>
#include <kbibtexeditlistbox.h>
#include "kbibtexentrywidget.h"

static const QString MonthsI18N[] =
    {
        i18n( "January" ), i18n( "February" ), i18n( "March" ), i18n( "April" ), i18n( "May" ), i18n( "June" ), i18n( "July" ), i18n( "August" ), i18n( "September" ), i18n( "October" ), i18n( "November" ), i18n( "December" )
    };

static const QString Months[] =
    {
        QString( "January" ), QString( "February" ), QString( "March" ), QString( "April" ), QString( "May" ), QString( "June" ), QString( "July" ), QString( "August" ), QString( "September" ), QString( "October" ), QString( "November" ), QString( "December" )
    };

static const QString MonthsTriple[] =
    {
        QString( "jan" ), QString( "feb" ), QString( "mar" ), QString( "apr" ), QString( "may" ), QString( "jun" ), QString( "jul" ), QString( "aug" ), QString( "sep" ), QString( "oct" ), QString( "nov" ), QString( "dec" )
    };

KBibTeXEntryWidget::KBibTeXEntryWidget( BibTeXEntry *entry, BibTeXFile *bibtexfile, QWidget *parent, const char *name )
        : QWidget( parent, name ), m_entry( entry ), m_bibtexfile( bibtexfile ), m_enableAllFields( false ), m_sourcehighlighting( TRUE )
{
    if ( !name )
        setName( "KBibTeXEntryWidgetUI" );
    deletedUserFields = new QStringList();

    setupGUI();
    getEntryData();
    getUserFields();
}


KBibTeXEntryWidget::~KBibTeXEntryWidget()
{
    delete deletedUserFields;
}

void KBibTeXEntryWidget::setupGUI()
{
    QSpacerItem * spacer = NULL;
    QLabel *label = NULL;
    QWidget *tab = NULL;
    QHBoxLayout *hBoxLayout = NULL;
    QGridLayout *gridLayout = NULL;

    QVBoxLayout *mainlayout = new QVBoxLayout( this, 1, 6, "mainlayout" );

    hBoxLayout = new QHBoxLayout( 0, 0, 6, "layout" );
    label = new QLabel( i18n( "E&ntry Type:" ), this );
    hBoxLayout->addWidget( label );
    kComboBoxEntryType = new KComboBox( FALSE, this, "kComboBoxEntryType" );
    label->setBuddy( kComboBoxEntryType );
    kComboBoxEntryType->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    hBoxLayout->addWidget( kComboBoxEntryType );
    spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    hBoxLayout->addItem( spacer );
    label = new QLabel( i18n( "&Identifier" ), this );
    hBoxLayout->addWidget( label );
    lineEditID = new QLineEdit( this, "lineEditID" );
    label->setBuddy( lineEditID );
    lineEditID->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    hBoxLayout->addWidget( lineEditID );
    mainlayout->addLayout( hBoxLayout );

    // ================================================================
    // Tab "Title"
    // ----------------------------------------------------------------

    QTabWidget* tabWidgetMain = new QTabWidget( this, "tabWidgetMain" );

    tab = new QWidget( tabWidgetMain, "tab_title" );
    gridLayout = new QGridLayout( tab, 1, 1, 11, 6, "gridLayout" );

    label = new QLabel( i18n( "&Title:" ), tab );
    lineEditTitle = new QLineEdit( tab, "lineEditTitle" );
    label->setBuddy( lineEditTitle );
    gridLayout->addWidget( label, 0, 0 );
    gridLayout->addWidget( lineEditTitle, 0, 1 );

    label = new QLabel( i18n( "&Book Title:" ), tab );
    lineEditBookTitle = new QLineEdit( tab, "lineEditBookTitle" );
    label->setBuddy( lineEditBookTitle );
    gridLayout->addWidget( label, 1, 0 );
    gridLayout->addWidget( lineEditBookTitle, 1, 1 );

    label = new QLabel( i18n( "&Series:" ), tab );
    lineEditSeries = new QLineEdit( tab, "lineEditSeries" );
    label->setBuddy( lineEditSeries );
    gridLayout->addWidget( label, 2, 0 );
    gridLayout->addWidget( lineEditSeries, 2, 1 );

    spacer = new QSpacerItem( 20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 3, 1 );

    tabWidgetMain->insertTab( tab, i18n( "Title" ) );

    // ================================================================
    // Tab "Author"
    // ----------------------------------------------------------------

    tab = new QWidget( tabWidgetMain, "tab_author" );
    hBoxLayout = new QHBoxLayout( tab, 11, 6, "hBoxLayout" );
    elbAuthor = new KBibTeXEditListBox( i18n( "&Author:" ), tab, "elbAuthor" );
    hBoxLayout->addWidget( elbAuthor );

    spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    hBoxLayout->addItem( spacer );

    elbEditor = new KBibTeXEditListBox( i18n( "&Editor:" ), tab, "elbEditor" );
    hBoxLayout->addWidget( elbEditor );
    tabWidgetMain->insertTab( tab, i18n( "Author" ) );

    // ================================================================
    // Tab "Publication"
    // ----------------------------------------------------------------

    tab = new QWidget( tabWidgetMain, "tab_publication" );
    gridLayout = new QGridLayout( tab, 8, 14, 11, 6, "gridLayout" );

    kLineEditHowPublished = new KLineEdit( tab, "kLineEditHowPublished" );
    kLineEditHowPublished->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditHowPublished, 0, 0, 1, 5 );
    label = new QLabel( i18n( "Ho&w Published:" ), tab, "textLabelHowPublished" );
    label->setBuddy( kLineEditHowPublished );
    gridLayout->addWidget( label, 0, 0 );

    kLineEditPublisher = new KLineEdit( tab, "kLineEditPublisher" );
    kLineEditPublisher->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditPublisher, 0, 0, 8, 12 );
    label = new QLabel( i18n( "&Publisher:" ), tab, "textLabelPublisher" );
    label->setBuddy( kLineEditPublisher );
    gridLayout->addWidget( label, 0, 7 );

    kLineEditOrganization = new KLineEdit( tab, "kLineEditOrganization" );
    kLineEditOrganization->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditOrganization, 1, 1, 1, 5 );
    label = new QLabel( i18n( "Org&anization" ), tab, "textLabelOrganization" );
    label->setBuddy( kLineEditOrganization );
    gridLayout->addWidget( label, 1, 0 );

    kLineEditInstitution = new KLineEdit( tab, "kLineEditInstitution" );
    kLineEditInstitution->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditInstitution, 1, 1, 8, 12 );
    label = new QLabel( i18n( "Instit&ution:" ), tab, "textLabelInstitution" );
    label->setBuddy( kLineEditInstitution );
    gridLayout->addWidget( label, 1, 7 );

    kLineEditSchool = new KLineEdit( tab, "kLineEditSchool" );
    kLineEditSchool->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditSchool, 2, 2, 1, 5 );
    label = new QLabel( i18n( "Schoo&l:" ), tab, "textLabelSchool" );
    label->setBuddy( kLineEditSchool );
    gridLayout->addWidget( label, 2, 0 );

    kLineEditAddress = new KLineEdit( tab, "kLineEditAddress" );
    kLineEditAddress->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditAddress, 2, 2, 8, 12 );
    label = new QLabel( i18n( "Add&ress:" ), tab, "textLabelAddress" );
    label->setBuddy( kLineEditAddress );
    gridLayout->addWidget( label, 2, 7 );

    kLineEditJournal = new KLineEdit( tab, "kLineEditJournal" );
    kLineEditJournal->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditJournal, 3, 3, 1, 5 );
    label = new QLabel( i18n( "&Journal:" ), tab, "textLabelJournal" );
    label->setBuddy( kLineEditJournal );
    gridLayout->addWidget( label, 3, 0 );

    lineEditPages = new QLineEdit( tab, "lineEditPages" );
    lineEditPages->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( lineEditPages, 3, 3, 8, 12 );
    label = new QLabel( i18n( "Pa&ges:" ), tab, "textLabelPages" );
    label->setBuddy( lineEditPages );
    gridLayout->addWidget( label, 3, 7 );

    kLineEditCrossRef = new KLineEdit( tab, "kLineEditCrossRef" );
    kLineEditCrossRef->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditCrossRef, 4, 4, 1, 5 );
    label = new QLabel( i18n( "Cro&ss Reference:" ), tab, "textLabelCrossRef" );
    label->setBuddy( kLineEditCrossRef );
    gridLayout->addWidget( label, 4, 0 );

    spinBoxEdition = new QSpinBox( tab, "spinBoxEdition" );
    spinBoxEdition->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    spinBoxEdition->setMaxValue( 500 );
    spinBoxEdition->setMinValue( 0 );
    gridLayout->addWidget( spinBoxEdition, 5, 1 );
    label = new QLabel( i18n( "E&dition:" ), tab, "textLabelEdition" );
    label->setBuddy( spinBoxEdition );
    gridLayout->addWidget( label, 5, 0 );

    kLineEditNumber = new KLineEdit( tab, "kLineEditNumber" );
    kLineEditNumber->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditNumber, 5, 5, 4, 9 );
    label = new QLabel( i18n( "Num&ber:" ), tab, "textLabelNumber" );
    label->setBuddy( kLineEditNumber );
    gridLayout->addWidget( label, 5, 3 );

    spinBoxVolume = new QSpinBox( tab, "spinBoxVolume" );
    spinBoxVolume->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    spinBoxVolume->setMaxValue( 5000 );
    spinBoxVolume->setMinValue( 0 );
    gridLayout->addWidget( spinBoxVolume, 5, 12 );
    label = new QLabel( i18n( "&Volume:" ), tab, "textLabelVolume" );
    label->setBuddy( spinBoxVolume );
    gridLayout->addWidget( label, 5, 11 );

    lineEditChapter = new QLineEdit( tab, "lineEditChapter" );
    lineEditChapter->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addWidget( lineEditChapter, 6, 1 );
    label = new QLabel( i18n( "Chap&ter:" ), tab, "textLabelChapter" );
    label->setBuddy( lineEditChapter );
    gridLayout->addWidget( label, 6, 0 );

    kComboBoxMonth = new KComboBox( FALSE, tab, "kComboBoxMonth" );
    kComboBoxMonth->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kComboBoxMonth, 6, 6, 4, 9 );
    label = new QLabel( i18n( "Mont&h:" ), tab, "textLabelMonth" );
    label->setBuddy( kComboBoxMonth );
    gridLayout->addWidget( label, 6, 3 );

    spinBoxYear = new QSpinBox( tab, "spinBoxYear" );
    spinBoxYear->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    spinBoxYear->setMaxValue( 4000 );
    spinBoxYear->setMinValue( 0 );
    spinBoxYear->setValue( 2000 );
    gridLayout->addWidget( spinBoxYear, 6, 12 );
    label = new QLabel( i18n( "&Year:" ), tab, "textLabelYear" );
    label->setBuddy( spinBoxYear );
    gridLayout->addWidget( label, 6, 11 );

    kLineEditISBN = new KLineEdit( tab, "kLineEditISBN" );
    kLineEditISBN->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditISBN, 7, 7, 1, 5 );
    label = new QLabel( i18n( "ISBN:" ), tab, "textLabelISBN" );
    label->setBuddy( kLineEditISBN );
    gridLayout->addWidget( label, 7, 0 );

    kLineEditISSN = new KLineEdit( tab, "kLineEditISSN" );
    kLineEditISSN->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    gridLayout->addMultiCellWidget( kLineEditISSN, 7, 7, 8, 12 );
    label = new QLabel( i18n( "ISSN:" ), tab, "textLabelISSN" );
    label->setBuddy( kLineEditISSN );
    gridLayout->addWidget( label, 7, 7 );

    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 0, 6 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 5, 2 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Maximum );
    gridLayout->addItem( spacer, 5, 10 );
    spacer = new QSpacerItem( 10, 10, QSizePolicy::Preferred, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 11, 0 );

    tabWidgetMain->insertTab( tab, i18n( "Publication" ) );

    // ================================================================
    // Tab "Misc"
    // ----------------------------------------------------------------

    tab = new QWidget( tabWidgetMain, "tab" );
    gridLayout = new QGridLayout( tab, 1, 1, 11, 6, "gridLayout" );

    lineEditType = new QLineEdit( tab, "lineEditType" );
    gridLayout->addWidget( lineEditType, 0, 1 );
    label = new QLabel( i18n( "&Type:" ), tab, "textLabelType" );
    label->setBuddy( lineEditType );
    gridLayout->addWidget( label, 0, 0 );

    label = new QLabel( i18n( "&Key:" ), tab, "textLabelKey" );
    gridLayout->addWidget( label, 0, 3 );
    lineEditKey = new QLineEdit( tab, "lineEditKey" );
    label->setBuddy( lineEditKey );
    gridLayout->addWidget( lineEditKey, 0, 4 );

    lineEditNote = new QLineEdit( tab, "lineEditNote" );
    gridLayout->addMultiCellWidget( lineEditNote, 1, 1, 1, 4 );
    label = new QLabel( i18n( "Not&e:" ), tab, "textLabelNote" );
    label->setBuddy( lineEditNote );
    gridLayout->addWidget( label, 1, 0 );

    spacer = new QSpacerItem( 24, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    gridLayout->addItem( spacer, 0, 2 );

    lineEditAnnote = new QLineEdit( tab, "lineEditAnnote" );
    gridLayout->addMultiCellWidget( lineEditAnnote, 2, 2, 1, 4 );
    label = new QLabel( i18n( "&Annote:" ), tab, "textLabelAnnote" );
    label->setBuddy( lineEditAnnote );
    gridLayout->addWidget( label, 2, 0 );

    QHBoxLayout* layout_2 = new QHBoxLayout( 0, 0, 6, "layout_2" );
    lineEditURL = new QLineEdit( tab, "lineEditURL" );
    layout_2->addWidget( lineEditURL );
    label = new QLabel( i18n( "&URL:" ), tab, "textLabelURL" );
    label->setBuddy( lineEditURL );
    gridLayout->addWidget( label, 3, 0 );
    kPushButtonURLtoNote = new KPushButton( i18n( "&Apply to field 'Note'" ), tab, "kPushButtonURLtoNote" );
    layout_2->addWidget( kPushButtonURLtoNote );
    gridLayout->addMultiCellLayout( layout_2, 3, 3, 1, 4 );

    kTextEditAbstract = new KTextEdit( tab, "kTextEditAbstract" );
    gridLayout->addMultiCellWidget( kTextEditAbstract, 4, 5, 1, 4 );
    label = new QLabel( i18n( "A&bstract:" ), tab, "textLabelAbstract" );
    label->setBuddy( kTextEditAbstract );
    gridLayout->addWidget( label, 4, 0 );

    spacer = new QSpacerItem( 20, 160, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 5, 0 );

    tabWidgetMain->insertTab( tab, i18n( "Misc" ) );

    // ================================================================
    // Tab "User Fields"
    // ----------------------------------------------------------------

    tab = new QWidget( tabWidgetMain, "tab" );
    gridLayout = new QGridLayout( tab, 1, 1, 11, 6, "gridLayout" );

    kPushButtonUserDelete = new KPushButton( tab, "kPushButtonUserDelete" );
    gridLayout->addWidget( kPushButtonUserDelete, 1, 2 );

    kHistoryComboUserName = new KHistoryCombo( tab, "kHistoryComboUserName" );
    gridLayout->addWidget( kHistoryComboUserName, 0, 1 );
    label = new QLabel( i18n( "&Key:" ), tab, "textLabelUserKey" );
    label->setBuddy( kHistoryComboUserName );
    gridLayout->addWidget( label, 0, 0 );

    kPushButtonUserAdd = new KPushButton( i18n( "&Add" ), tab, "kPushButtonUserAdd" );
    gridLayout->addWidget( kPushButtonUserAdd, 0, 2 );

    spacer = new QSpacerItem( 20, 110, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 2, 0 );

    spacer = new QSpacerItem( 20, 110, QSizePolicy::Minimum, QSizePolicy::Expanding );
    gridLayout->addItem( spacer, 2, 2 );

    kListViewUserFields = new KListView( tab, "kListViewUserFields" );
    kListViewUserFields->addColumn( i18n( "Key" ) );
    kListViewUserFields->addColumn( i18n( "Value" ) );
    gridLayout->addMultiCellWidget( kListViewUserFields, 3, 3, 0, 2 );

    kTextEditUserValue = new KTextEdit( tab, "kTextEditUserValue" );
    gridLayout->addMultiCellWidget( kTextEditUserValue, 1, 2, 1, 1 );
    label = new QLabel( i18n( "&Value:" ), tab, "textLabelUserValue" );
    label->setBuddy( kTextEditUserValue );
    gridLayout->addWidget( label, 1, 0 );

    tabWidgetMain->insertTab( tab, i18n( "User Fields" ) );

    // ================================================================
    // Tab "Source"
    // ----------------------------------------------------------------

    sourceTab = new QWidget( tabWidgetMain, "tab_source" );
    gridLayout = new QGridLayout( sourceTab, 2, 1, 11, 6, "gridLayout" );

    checkBoxSourceHighlightning = new QCheckBox( i18n( "&Highlight source" ), sourceTab, "checkBoxSourceHighlightning" );
    checkBoxSourceHighlightning->setChecked( m_sourcehighlighting );
    gridLayout->addWidget( checkBoxSourceHighlightning, 0, 0 );

    textEditSource = new QTextEdit( sourceTab );
    textEditSource->setReadOnly( TRUE );
    gridLayout->addWidget( textEditSource, 1, 0 );

    tabWidgetMain->insertTab( sourceTab, i18n( "Source" ) );


    mainlayout->addWidget( tabWidgetMain );
    resize( QSize( 768, 480 ).expandedTo( minimumSizeHint() ) );
    clearWState( WState_Polished );

    // tab order
    setTabOrder( kComboBoxEntryType, lineEditID );
    setTabOrder( lineEditID, tabWidgetMain );
    setTabOrder( tabWidgetMain, lineEditTitle );
    setTabOrder( lineEditTitle, lineEditBookTitle );
    setTabOrder( lineEditBookTitle, elbAuthor );
    setTabOrder( elbAuthor, elbEditor );
    setTabOrder( elbEditor, lineEditSeries );
    setTabOrder( lineEditSeries, kLineEditHowPublished );
    setTabOrder( kLineEditHowPublished, kLineEditPublisher );
    setTabOrder( kLineEditPublisher, kLineEditOrganization );
    setTabOrder( kLineEditOrganization, kLineEditInstitution );
    setTabOrder( kLineEditInstitution, kLineEditSchool );
    setTabOrder( kLineEditSchool, kLineEditAddress );
    setTabOrder( kLineEditAddress, kLineEditJournal );
    setTabOrder( kLineEditJournal, lineEditPages );
    setTabOrder( lineEditPages, kLineEditCrossRef );
    setTabOrder( kLineEditCrossRef, spinBoxEdition );
    setTabOrder( spinBoxEdition, kLineEditNumber );
    setTabOrder( kLineEditNumber, spinBoxVolume );
    setTabOrder( spinBoxVolume, lineEditChapter );
    setTabOrder( lineEditChapter, kComboBoxMonth );
    setTabOrder( kComboBoxMonth, spinBoxYear );
    setTabOrder( spinBoxYear, kLineEditISBN );
    setTabOrder( kLineEditISBN, kLineEditISSN );
    setTabOrder( kLineEditISSN, lineEditType );
    setTabOrder( lineEditType, lineEditKey );
    setTabOrder( lineEditKey, lineEditNote );
    setTabOrder( lineEditNote, lineEditAnnote );
    setTabOrder( lineEditAnnote, lineEditURL );
    setTabOrder( lineEditURL, kPushButtonURLtoNote );
    setTabOrder( kPushButtonURLtoNote, kTextEditAbstract );
    setTabOrder( kTextEditAbstract, kHistoryComboUserName );
    setTabOrder( kHistoryComboUserName, kPushButtonUserAdd );
    setTabOrder( kPushButtonUserAdd, kTextEditUserValue );
    setTabOrder( kTextEditUserValue, kPushButtonUserDelete );
    setTabOrder( kPushButtonUserDelete, kListViewUserFields );


    elbAuthor->setButtonText( KBibTeXEditListBox::Add, i18n( "Add Au&thor" ) );
    elbAuthor->setButtonText( KBibTeXEditListBox::Modify, i18n( "Modify" ) );
    elbAuthor->setButtonText( KBibTeXEditListBox::Delete, i18n( "Remove" ) );
    elbAuthor->setButtonText( KBibTeXEditListBox::Up, i18n( "&Up" ) );
    elbAuthor->setButtonText( KBibTeXEditListBox::Down, i18n( "Do&wn" ) );
    elbEditor->setButtonText( KBibTeXEditListBox::Add, i18n( "Add Edito&r" ) );
    elbEditor->setButtonText( KBibTeXEditListBox::Modify, i18n( "Modify" ) );
    elbEditor->setButtonText( KBibTeXEditListBox::Delete, i18n( "Remove" ) );
    elbEditor->setButtonText( KBibTeXEditListBox::Up, i18n( "U&p" ) );
    elbEditor->setButtonText( KBibTeXEditListBox::Down, i18n( "Dow&n" ) );
    kPushButtonUserAdd->setEnabled( FALSE );
    kPushButtonUserDelete->setEnabled( FALSE );
    kListViewUserFields->setAllColumnsShowFocus( true );

    spinBoxEdition->setMinValue( 0 );
    spinBoxEdition->setMaxValue( 10000 );
    spinBoxEdition->setSpecialValueText( i18n( "not set" ) );
    spinBoxVolume->setMinValue( 0 );
    spinBoxVolume->setMaxValue( 10000 );
    spinBoxVolume->setSpecialValueText( i18n( "not set" ) );
    spinBoxYear->setMinValue( 999 );
    spinBoxYear->setMaxValue( 3000 );
    spinBoxYear->setSpecialValueText( i18n( "not set" ) );

    kComboBoxMonth->insertItem( i18n( "not set" ) );
    for ( int i = 0; i < 12; i++ )
        kComboBoxMonth->insertItem( MonthsI18N[ i ] );

    setupEntryTypes();

    connect( kComboBoxEntryType, SIGNAL( activated( int ) ), this, SLOT( slotEntryTypeChanged( int ) ) );
    connect( kListViewUserFields, SIGNAL( executed( QListViewItem* ) ), this, SLOT( slotUserFieldExecute( QListViewItem* ) ) );
    connect( kHistoryComboUserName, SIGNAL( textChanged( const QString& ) ), this, SLOT( slotUserNameChanged( const QString& ) ) );
    connect( kPushButtonUserAdd, SIGNAL( clicked( ) ), this, SLOT( slotUserAddClicked( ) ) );
    connect( kPushButtonUserDelete, SIGNAL( clicked( ) ), this, SLOT( slotUserDeleteClicked( ) ) );
    connect( kPushButtonURLtoNote, SIGNAL( clicked() ), this, SLOT( slotApplyURLtoNode() ) );
    //    connect( pushButtonJournal, SIGNAL( clicked( ) ), this, SLOT( slotSelectStringJournal() ) );
    connect( tabWidgetMain, SIGNAL( currentChanged( QWidget* ) ), this, SLOT( slotCurrentTabChanged( QWidget* ) ) );
    connect( checkBoxSourceHighlightning, SIGNAL( toggled( bool ) ), this, SLOT( slotSourceHighlightningToggle( bool ) ) );
}

void KBibTeXEntryWidget::setupEntryTypes()
{
    int i;
    BibTeXElement::ElementType elementType;

    i = ( int ) BibTeXEntry::etArticle;
    elementType = ( BibTeXEntry::ElementType ) i;
    while ( elementType != BibTeXEntry::etUnknown )
    {
        kComboBoxEntryType->insertItem( BibTeXElement::elementTypeToString( elementType ) );
        elementType = ( BibTeXElement::ElementType ) ++i;
    }
}

void KBibTeXEntryWidget::getEntryData()
{
    bool ok;
    BibTeXEntry::Field *field;

    kComboBoxEntryType->setCurrentItem( m_entry->elementType() - ( int ) BibTeXEntry::etArticle );
    lineEditID->setText( m_entry->id() );
    field = m_entry->field( BibTeXEntry::ftTitle );
    if ( field )
        lineEditTitle->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftBookTitle );
    if ( field )
        lineEditBookTitle->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftSeries );
    if ( field )
        lineEditSeries->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftHowPublished );
    if ( field )
        kLineEditHowPublished->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftPublisher );
    if ( field )
        kLineEditPublisher->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftOrganization );
    if ( field )
        kLineEditOrganization->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftInstitution );
    if ( field )
        kLineEditInstitution->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftSchool );
    if ( field )
        kLineEditSchool->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftAddress );
    if ( field )
        kLineEditAddress->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftJournal );
    if ( field )
        kLineEditJournal->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftCrossRef );
    if ( field )
        kLineEditCrossRef->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftISBN );
    if ( field )
        kLineEditISBN->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftISSN );
    if ( field )
        kLineEditISSN->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftPages );
    if ( field )
        lineEditPages->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftNumber );
    if ( field )
        kLineEditNumber->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftChapter );
    if ( field )
        lineEditChapter->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftType );
    if ( field )
        lineEditType->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftKey );
    if ( field )
        lineEditKey->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftNote );
    if ( field )
        lineEditNote->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftAnnote );
    if ( field )
        lineEditAnnote->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftAbstract );
    if ( field )
        kTextEditAbstract->setText( field->value );
    field = m_entry->field( BibTeXEntry::ftURL );
    if ( field )
        lineEditURL->setText( field->value );

    field = m_entry->field( BibTeXEntry::ftEdition );
    int edition = 0;
    if ( field && ( ( edition = field->value.toInt( &ok ) ) ) && ok && ( edition >= 1 ) && ( edition <= spinBoxEdition->maxValue() ) )
        spinBoxEdition->setValue( edition );
    else
        spinBoxEdition->setValue( spinBoxEdition->minValue() );

    field = m_entry->field( BibTeXEntry::ftVolume );
    int volume = 0;
    if ( field && ( ( volume = field->value.toInt( &ok ) ) ) && ok && ( volume >= 1 ) && ( volume <= spinBoxVolume->maxValue() ) )
        spinBoxVolume->setValue( volume );
    else
        spinBoxVolume->setValue( spinBoxVolume->minValue() );

    field = m_entry->field( BibTeXEntry::ftYear );
    int year = 0;
    if ( field && ( ( year = field->value.toInt( &ok ) ) ) && ok && ( year >= 1000 ) && ( year <= spinBoxYear->maxValue() ) )
        spinBoxYear->setValue( year );
    else
        spinBoxYear->setValue( spinBoxYear->minValue() );

    field = m_entry->field( BibTeXEntry::ftAuthor );
    if ( field )
        elbAuthor->setLines( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field->value ) );
    field = m_entry->field( BibTeXEntry::ftEditor );
    if ( field )
        elbEditor->setLines( QStringList::split( QRegExp( "\\s+(,|and|&)\\s+", false ), field->value ) );

    field = m_entry->field( BibTeXEntry::ftMonth );
    if ( field )
        for ( int i = 0; i < 12; i++ )
        {
            QString entryMonth = field ->value.lower();
            if ( ( entryMonth.compare( Months[ i ].lower() ) == 0 ) || ( entryMonth.compare( MonthsTriple[ i ].lower() ) == 0 ) || ( entryMonth.compare( MonthsI18N[ i ].lower() ) == 0 ) )
                kComboBoxMonth->setCurrentItem( i + 1 );
        }

    changeElementType( m_entry->elementType() );
}

bool KBibTeXEntryWidget::setEntryDataSpinBox( BibTeXEntry *entry, QSpinBox *sbx, BibTeXEntry::FieldType fieldType )
{
    if ( sbx->value() > sbx->minValue() )
    {
        QString text = QString::number( sbx->value() );
        entry->setField( fieldType, text );
        return true;
    }
    else
    {
        entry->deleteField( fieldType );
        return false;
    }
}

bool KBibTeXEntryWidget::setEntryDataText( BibTeXEntry *entry, QString text, BibTeXEntry::FieldType fieldType )
{
    if ( !text.isEmpty() )
        entry->setField( fieldType, text );
    else
        entry->deleteField( fieldType );

    return ( BibTeXEntry::getRequireStatus( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ), fieldType ) != BibTeXEntry::frsRequired ) || !text.isEmpty();
}

bool KBibTeXEntryWidget::setEntryData( BibTeXEntry *entry )
{
    bool result = true;
    QString text;

    entry->setElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ) );
    text = lineEditID->text();
    entry->setId( text );

    setEntryDataText( entry, lineEditTitle->text(), BibTeXEntry::ftTitle );
    setEntryDataText( entry, lineEditBookTitle->text(), BibTeXEntry::ftBookTitle );
    setEntryDataText( entry, lineEditSeries->text(), BibTeXEntry::ftSeries );

    setEntryDataText( entry, kLineEditHowPublished->text(), BibTeXEntry::ftHowPublished );
    setEntryDataText( entry, kLineEditPublisher->text(), BibTeXEntry::ftPublisher );
    setEntryDataText( entry, kLineEditOrganization->text(), BibTeXEntry::ftOrganization );
    setEntryDataText( entry, kLineEditInstitution->text(), BibTeXEntry::ftInstitution );
    setEntryDataText( entry, kLineEditSchool->text(), BibTeXEntry::ftSchool );
    setEntryDataText( entry, kLineEditAddress->text(), BibTeXEntry::ftAddress );
    setEntryDataText( entry, kLineEditJournal->text(), BibTeXEntry::ftJournal );
    setEntryDataText( entry, kLineEditCrossRef->text(), BibTeXEntry::ftCrossRef );
    setEntryDataText( entry, kLineEditISBN->text(), BibTeXEntry::ftISBN );
    setEntryDataText( entry, kLineEditISSN->text(), BibTeXEntry::ftISSN );
    setEntryDataText( entry, lineEditPages->text(), BibTeXEntry::ftPages );
    setEntryDataText( entry, lineEditChapter->text(), BibTeXEntry::ftChapter );
    setEntryDataText( entry, kLineEditNumber->text(), BibTeXEntry::ftNumber );
    setEntryDataText( entry, lineEditType->text(), BibTeXEntry::ftType );
    setEntryDataText( entry, lineEditKey->text(), BibTeXEntry::ftKey );
    setEntryDataText( entry, lineEditNote->text(), BibTeXEntry::ftNote );
    setEntryDataText( entry, lineEditAnnote->text(), BibTeXEntry::ftAnnote );
    setEntryDataText( entry, kTextEditAbstract->text(), BibTeXEntry::ftAbstract );
    setEntryDataText( entry, lineEditURL->text(), BibTeXEntry::ftURL );

    setEntryDataSpinBox( entry, spinBoxEdition, BibTeXEntry::ftEdition );
    setEntryDataSpinBox( entry, spinBoxVolume, BibTeXEntry::ftVolume );
    setEntryDataSpinBox( entry, spinBoxYear, BibTeXEntry::ftYear );

    setEntryDataText( entry, elbAuthor->lines().join( " and " ), BibTeXEntry::ftAuthor );
    setEntryDataText( entry, elbEditor->lines().join( " and " ), BibTeXEntry::ftEditor );

    if ( kComboBoxMonth->currentItem() > 0 )
    {
        QString text = QString( MonthsTriple[ kComboBoxMonth->currentItem() - 1 ] );
        entry->setField( BibTeXEntry::ftMonth, text );
    }
    else
        entry->deleteField( BibTeXEntry::ftMonth );

    for ( QStringList::iterator it = deletedUserFields->begin(); it != deletedUserFields->end(); it++ )
        entry->deleteField( *it );
    for ( QListViewItemIterator it( kListViewUserFields ); it.current(); it++ )
    {
        entry->deleteField( it.current() ->text( 0 ) );
        entry->setField( it.current() ->text( 0 ), it.current() ->text( 1 ) );
    }

    return result;
}

QDialog::DialogCode KBibTeXEntryWidget::execute( BibTeXEntry *entry, BibTeXFile *bibtexfile, const QString& caption, QWidget *parent, const char *name )
{
    KDialogBase * dlg = new KDialogBase( parent, name, true, caption, KDialogBase::Ok | KDialogBase::Cancel );
    QVBox *vbox = new QVBox( dlg );
    KBibTeXEntryWidget* ui = new KBibTeXEntryWidget( entry, bibtexfile, vbox, "kbibtexentrywidget" );
    connect( dlg, SIGNAL( okClicked() ), ui, SLOT( slotOk() ) );

    QCheckBox *cb = new QCheckBox( i18n( "Enable all &fields for editing" ), vbox );
    cb->setChecked( kbibtexsettings.enableallfields );
    connect( cb, SIGNAL( toggled( bool ) ), ui, SLOT( slotEnableAllFields( bool ) ) );

    dlg->setMainWidget( vbox );
    QDialog::DialogCode result = ( QDialog::DialogCode ) dlg->exec();

    delete( ui );
    delete( dlg );

    return result;
}

void KBibTeXEntryWidget::slotOk()
{
    setEntryData( m_entry );
}

void KBibTeXEntryWidget::changeElementType( BibTeXElement::ElementType elementType )
{
    setWidgetStatus( lineEditTitle, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftTitle ) );
    setWidgetStatus( lineEditBookTitle, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftBookTitle ) );
    setWidgetStatus( lineEditSeries, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftSeries ) );
    setWidgetStatus( elbAuthor, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAuthor ) );
    setWidgetStatus( elbEditor, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftEditor ) );
    setWidgetStatus( kLineEditHowPublished, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftHowPublished ) );
    setWidgetStatus( kLineEditOrganization, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftOrganization ) );
    setWidgetStatus( kLineEditSchool, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftSchool ) );
    setWidgetStatus( kLineEditJournal, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftJournal ) );
    //    setWidgetStatus( pushButtonJournal, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftJournal ) );
    setWidgetStatus( kLineEditCrossRef, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftCrossRef ) );
    setWidgetStatus( kLineEditPublisher, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftPublisher ) );
    setWidgetStatus( kLineEditInstitution, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftInstitution ) );
    setWidgetStatus( kLineEditAddress, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAddress ) );
    setWidgetStatus( kLineEditISBN, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftISBN ) );
    setWidgetStatus( kLineEditISSN, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftISSN ) );
    setWidgetStatus( lineEditPages, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftPages ) );
    setWidgetStatus( spinBoxEdition, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftEdition ) );
    setWidgetStatus( kLineEditNumber, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftNumber ) );
    setWidgetStatus( spinBoxVolume, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftVolume ) );
    setWidgetStatus( lineEditChapter, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftChapter ) );
    setWidgetStatus( kComboBoxMonth, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftMonth ) );
    setWidgetStatus( spinBoxYear, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftYear ) );
    setWidgetStatus( lineEditType, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftType ) );
    setWidgetStatus( lineEditKey, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftKey ) );
    setWidgetStatus( lineEditNote, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftNote ) );
    setWidgetStatus( lineEditAnnote, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftAnnote ) );
    //setWidgetStatus( lineEditURL, BibTeXEntry::getRequireStatus( elementType, BibTeXEntry::ftURL ) );
}

void KBibTeXEntryWidget::setWidgetStatus( QWidget* widget, BibTeXEntry::FieldRequireStatus requiredStatus )
{
    widget->setEnabled( ( requiredStatus != BibTeXEntry::frsIgnored ) || m_enableAllFields );

    QWidget *parent = widget->parentWidget();
    for ( QObjectListIterator it( *( parent->children() ) ); it.current(); ++it )
    {
        QLabel *neighbour = dynamic_cast<QLabel*>( it.current() );
        if ( neighbour && ( neighbour->buddy() == widget ) )
        {
            QFont font = parent->font();
            switch ( requiredStatus )
            {
            case BibTeXEntry::frsRequired:
                font.setBold( true );
                break;
            default:
                break;
            }
            neighbour->setFont( font );
        }
    }
}

void KBibTeXEntryWidget::slotEntryTypeChanged( int index )
{
    changeElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + index ) );
}

void KBibTeXEntryWidget::slotEnableAllFields( bool enable )
{
    m_enableAllFields = enable;
    changeElementType( ( BibTeXElement::ElementType ) ( ( int ) BibTeXEntry::etArticle + kComboBoxEntryType->currentItem() ) );
}

void KBibTeXEntryWidget::getUserFields()
{
    for ( QValueList<BibTeXEntry::Field*>::iterator it = m_entry->begin(); it != m_entry->end(); it++ )
    {
        if ( ( *it ) ->fieldType == BibTeXEntry::ftUnknown )
            new KBibTeXFieldListViewItem( ( *it ) ->fieldType, ( *it ) ->key, ( *it ) ->value, kListViewUserFields );
    }
}

void KBibTeXEntryWidget::slotUserFieldExecute( QListViewItem* item )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( item );
    kHistoryComboUserName->setCurrentText( flvi->fieldName() );
    kTextEditUserValue->setText( flvi->fieldValue() );
}

void KBibTeXEntryWidget::slotUserNameChanged( const QString& newText )
{
    QListViewItem * item = kListViewUserFields->findItem( newText, 0 ) ;
    bool contains = item != NULL;
    bool isUnknown = BibTeXEntry::fieldTypeFromString( newText ) == BibTeXEntry::ftUnknown;

    if ( contains )
        kListViewUserFields->setSelected( item, TRUE );
    kPushButtonUserDelete->setEnabled( contains );
    kPushButtonUserAdd->setEnabled( !newText.isEmpty() && isUnknown );
    kPushButtonUserAdd->setText( contains ? i18n( "&Apply" ) : i18n( "&Add" ) );
}

void KBibTeXEntryWidget::slotUserAddClicked( )
{
    KBibTeXFieldListViewItem * flvi = dynamic_cast<KBibTeXFieldListViewItem*>( kListViewUserFields->findItem( kHistoryComboUserName->currentText(), 0 ) );

    if ( flvi != NULL )
    {
        flvi->setFieldValue( kTextEditUserValue->text() );
        flvi->updateItem();
    }
    else
        kListViewUserFields->setSelected( new KBibTeXFieldListViewItem( BibTeXEntry::fieldTypeFromString( kHistoryComboUserName->currentText() ), kHistoryComboUserName->currentText(), kTextEditUserValue->text(), kListViewUserFields ), true );

    kPushButtonUserAdd->setText( i18n( "&Apply" ) );
    kPushButtonUserDelete->setEnabled( TRUE );
}

void KBibTeXEntryWidget::slotUserDeleteClicked()
{
    QListViewItem * item = kListViewUserFields->findItem( kHistoryComboUserName->currentText(), 0 );

    if ( item != NULL )
    {
        deletedUserFields->append( item->text( 0 ) );
        kListViewUserFields->removeItem( item );
        kPushButtonUserAdd->setText( i18n( "&Add" ) );
        kPushButtonUserDelete->setEnabled( FALSE );
    }
}

void KBibTeXEntryWidget::slotApplyURLtoNode()
{
    lineEditNote->setText( "\\url{" + lineEditURL->text() + "}" );
}

void KBibTeXEntryWidget::slotSelectStringJournal()
{
    QString macro = KBibTeXSelectString::execute( m_bibtexfile, this );
    if ( !macro.isNull() )
        kLineEditJournal->setText( macro );
}

void KBibTeXEntryWidget::slotCurrentTabChanged( QWidget* newTab )
{
    if ( newTab == sourceTab )
        refreshSourceView();
}

void KBibTeXEntryWidget::slotSourceHighlightningToggle( bool highlight )
{
    m_sourcehighlighting = highlight;
    refreshSourceView();
}

void KBibTeXEntryWidget::refreshSourceView()
{
    BibTeXEntry * tmpEntry = new BibTeXEntry( m_entry );
    setEntryData( tmpEntry );
    QString display = QString( "<qt><pre>%1</pre></qt>" ).arg( tmpEntry->toBibTeXString( BibTeXFile::encUTF8, m_sourcehighlighting ) );
    textEditSource->setText( display );
    delete tmpEntry;
}

#include "kbibtexentrywidget.moc"
