/***************************************************************************
*   Copyright (C) 2004 by Thomas Fischer                                  *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qhbuttongroup.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <klocale.h>
#include <kdialog.h>

#include "settings.h"
#include "kbibtexsettingsfileio.h"

static const QString LanguagesI18N[] =
    {
        i18n( "English" ), i18n( "German" ), i18n( "Swedish" )
    };

static const QString Languages[] =
    {
        QString( "english" ), QString( "german" ), QString( "swedish" )
    };

static const QString BibliographyStyles[] =
    {
        QString( "plain" ), QString( "siam" ), QString( "alpha" ), QString( "apalike" ), QString( "abbrv" ), QString( "unsrt" ), QString( "ieeetr" )
    };


KBibTeXSettingsFileIO::KBibTeXSettingsFileIO( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    QVBoxLayout * layout = new QVBoxLayout( this, 0, KDialog::spacingHint() );

    bgEncoding = new QHButtonGroup( i18n( "Encoding of non-ASCII characters" ), this );
    rbEncodingLaTeX = new QRadioButton( i18n( "&LaTeX style" ), bgEncoding );
    bgEncoding->insert( rbEncodingLaTeX );
    rbEncodingUTF8 = new QRadioButton( i18n( "&UTF-8" ), bgEncoding );
    bgEncoding->insert( rbEncodingUTF8 );
    layout->addWidget( bgEncoding );

    QGroupBox *groupBox = new QGroupBox( 2, Horizontal, i18n( "Exporting" ), this );
    QLabel *label = new QLabel( i18n( "&Language:" ), groupBox );
    cbLanguage = new QComboBox( groupBox );
    label->setBuddy( cbLanguage );
    label = new QLabel( i18n( "&Biblography style:" ), groupBox );
    cbBibliographyStyle = new QComboBox( TRUE, groupBox );
    label->setBuddy( cbBibliographyStyle );
    layout->addWidget( groupBox );

    layout->addStretch( 1 );

    for ( unsigned int i = 0; i < sizeof( LanguagesI18N ) / sizeof( LanguagesI18N[ 0 ] ); i++ )
        cbLanguage->insertItem( LanguagesI18N[ i ] );
    for ( unsigned int i = 0; i < sizeof( BibliographyStyles ) / sizeof( BibliographyStyles[ 0 ] ); i++ )
        cbBibliographyStyle->insertItem( BibliographyStyles[ i ] );

    connect( bgEncoding, SIGNAL( clicked( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( cbLanguage, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
    connect( cbBibliographyStyle, SIGNAL( activated( int ) ), this, SLOT( slotConfigChanged() ) );
}


KBibTeXSettingsFileIO::~KBibTeXSettingsFileIO()
{
    //TODO
}

void KBibTeXSettingsFileIO::applyData()
{
    if ( rbEncodingLaTeX->isChecked() )
        kbibtexsettings.encoding = Settings::encLaTeX;
    else if ( rbEncodingUTF8->isChecked() )
        kbibtexsettings.encoding = Settings::encUTF8;
    kbibtexsettings.exportlanguage = Languages[ cbLanguage->currentItem() ];
    kbibtexsettings.exportbibliographystyle = cbBibliographyStyle->currentText();
}

void KBibTeXSettingsFileIO::setData()
{
    rbEncodingLaTeX->setChecked( kbibtexsettings.encoding == Settings::encLaTeX );
    rbEncodingUTF8->setChecked( kbibtexsettings.encoding == Settings::encUTF8 );

    for ( unsigned int i = 0; i < sizeof( LanguagesI18N ) / sizeof( LanguagesI18N[ 0 ] ); i++ )
        if ( Languages[ i ].compare( kbibtexsettings.exportlanguage ) == 0 )
        {
            cbLanguage->setCurrentItem( i );
            break;
        }

    cbBibliographyStyle->setCurrentText( kbibtexsettings.exportbibliographystyle );
}

void KBibTeXSettingsFileIO::slotConfigChanged()
{
    emit configChanged();
}

#include "kbibtexsettingsfileio.moc"
